@@ -1,5 +1,8 @@
 import logging
 import hashlib
+from collections import defaultdict
+connection_container_type = lambda y=lambda x=list:defaultdict(x):defaultdict(y)
+
 from .Exceptions import LifeLineChartNotEnoughInformationToDisplay
 
 logging.basicConfig()  # level=20)
@@ -32,6 +35,12 @@ class InstanceContainer():
         self._individual_constructor = individual_constructor
         self.instantiate_all = instantiate_all
         self.ancestor_width_cache = {}
+        self.connection_container = {}
+        self.connection_container.update({'i': connection_container_type(), 'f': connection_container_type()})
+        self.color_getters = {
+            'unique': self.color_generator_unique,
+            'surname': self.color_generator_surname
+        }
 
     def __iter__(self):  # iterate over all keys
         for type_id, instance in self._data.keys():
@@ -62,22 +71,26 @@ class InstanceContainer():
         if key[1] is None:
             return self._data[key]
         elif key[0] == 'i':
-            item = self._data.get(key)
-            if item is None:
+            item = self._data.get(key,'none')
+            if item == 'none':
                 try:
                     item = self._individual_constructor(self, key)
                     self._data[key] = item
-                except LifeLineChartNotEnoughInformationToDisplay:
+                except LifeLineChartNotEnoughInformationToDisplay as e:
+                    logger.info(str(e))
                     item = None
+                    self._data[key] = item
             return item
         elif key[0] == 'f':
-            item = self._data.get(key)
-            if item is None:
+            item = self._data.get(key,'none')
+            if item == 'none':
                 try:
                     item = self._family_constructor(self, key)
                     self._data[key] = item
-                except LifeLineChartNotEnoughInformationToDisplay:
+                except LifeLineChartNotEnoughInformationToDisplay as e:
+                    logger.info(str(e))
                     item = None
+                    self._data[key] = item
             return item
         return None
 
@@ -90,15 +103,23 @@ class InstanceContainer():
         """
         self._data.clear()
         self._data.update({('i', None): None, ('f', None): None})
+        self.ancestor_width_cache.clear()
+        self.clear_connections()
 
-    def color_generator(self, individual):
+    def clear_connections(self):
+        self.connection_container.clear()
+        self.connection_container.update({'i': connection_container_type(), 'f': connection_container_type()})
+
+    def color_generator_unique(self, gr_individual):
         """
         generate color for an individual
 
         Args:
-            individual (BaseIndividual): individual to generate a color for
+            individual (GraphicalIndividual): individual to generate a color for
         """
-        i = int(hashlib.sha1(individual.plain_name.encode(
+        individual = gr_individual.individual
+        seed = individual.plain_name + str(individual.individual_id)
+        i = int(hashlib.sha1(seed.encode(
             'utf8')).hexdigest(), 16) % (10 ** 8)
         c = (i*23 % 255, i*41 % 255, (i*79 % 245) + 10)
         f = 255/max(c)
@@ -106,6 +127,47 @@ class InstanceContainer():
         f = min(1, 500/sum(c))
         return [int(x*f) for x in c]
 
+    def color_generator_surname(self, gr_individual):
+        """
+        generate color for an individual
+
+        Args:
+            individual (GraphicalIndividual): individual to generate a color for
+        """
+        individual = gr_individual.individual
+        seed = ""
+        try:
+            seed = "".join(individual.get_name()[1:])
+        except Exception as e:
+            pass
+        i = int(hashlib.sha1(seed.encode(
+            'utf8')).hexdigest(), 16) % (10 ** 8)
+        c = (i*23 % 255, i*41 % 255, (i*79 % 245) + 10)
+        f = 255/max(c)
+        c = [int(x*f) for x in c]
+        f = min(1, 500/sum(c))
+        return [int(x*f) for x in c]
+
+    def color_generator_fathers_have_the_same_color(self, gr_individual):
+        """
+        generate color for an individual
+
+        Args:
+            individual (GraphicalIndividual): individual to generate a color for
+        """
+        parent_family, spouse_family = gr_individual.ancestor_chart_parent_family_placement
+        gr_husb = None
+        while parent_family is not None and parent_family.gr_husb is not None:
+            gr_husb = parent_family.gr_husb
+            parent_family, spouse_family = gr_husb.ancestor_chart_parent_family_placement
+        if gr_husb:
+            if gr_husb.color == (0,0,0):
+                gr_husb.color = self.color_getter(gr_husb)
+            return gr_husb.color
+        return self.color_getter(gr_individual)
+
+    color_getter = color_generator_unique
+
     def display_plain_name(self, individual):
         return ' '.join([n.strip() for n in individual.get_name() if n.strip() != ''])
 
@@ -153,10 +215,12 @@ class InstanceContainer():
 
         event = individual.events['birth_or_christening']
         if event:
-            if event['comment']:
-                return str(event['date'].date().year)
-            else:
+            if event['precision'] == 'dmy':
                 return event['date'].date().strftime('%d.%m.%Y')
+            elif event['precision'] == 'my':
+                return event['date'].date().strftime('%m.%Y')
+            else:
+                return str(event['date'].date().year)
         return None
 
     def display_death_date(self, individual):
@@ -169,10 +233,12 @@ class InstanceContainer():
 
         event = individual.events['death_or_burial']
         if event:
-            if event['comment']:
-                return str(event['date'].date().year)
-            else:
+            if event['precision'] == 'dmy':
                 return event['date'].date().strftime('%d.%m.%Y')
+            elif event['precision'] == 'my':
+                return event['date'].date().strftime('%m.%Y')
+            else:
+                return str(event['date'].date().year)
         return None
 
     def display_marriage_date(self, family):
@@ -184,10 +250,12 @@ class InstanceContainer():
         """
 
         event = family.marriage
-        if event['comment']:
-            return str(event['date'].date().year)
-        else:
+        if event['precision'] == 'dmy':
             return event['date'].date().strftime('%d.%m.%Y')
+        elif event['precision'] == 'my':
+            return event['date'].date().strftime('%m.%Y')
+        else:
+            return str(event['date'].date().year)
 
     def display_marriage_location(self, family):
         """
